import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:hw_introduction_screen/src/HwIntroductionButton.dart';
import 'package:hw_introduction_screen/src/HwIntroductionIndicator.dart';
import 'package:hw_introduction_screen/src/HwIntroductionItem.dart';
import 'package:hw_introduction_screen/src/HwIntroductionStyle.dart';

class HwIntroductionScreen extends StatefulWidget {
  final Widget? nextButton;
  final Color? nextButtonColor;

  final Widget? skipButton;
  final Color? skipButtonColor;

  final Widget? finishButton;
  final Color? finishButtonColor;

  final Color? indicatorActiveColor;
  final Color? indicatorUnactiveColor;

  final List<HwIntroductionItem> items;
  final Function? onFinish;

  HwIntroductionScreen({
    Key? key,
    this.nextButton,
    this.nextButtonColor,
    this.skipButton,
    this.skipButtonColor,
    this.finishButton,
    this.finishButtonColor,
    this.indicatorActiveColor,
    this.indicatorUnactiveColor,
    this.onFinish,
    required this.items,
  }) : super(key: key);

  _HwIntroductionScreenState createState() => _HwIntroductionScreenState();
}

class _HwIntroductionScreenState extends State<HwIntroductionScreen>
    with SingleTickerProviderStateMixin {
  int currentIndex = 0;
  bool isLoading = false;
  bool visibleSkipOrNextButton = true;
  bool visibleGetStartedButton = false;

  // Animation
  late final AnimationController _controllerAnimation = AnimationController(
      duration: new Duration(milliseconds: 300), vsync: this);

  late final Animation<Offset> _offsetAnimation =
      Tween<Offset>(begin: Offset.zero, end: const Offset(1.5, 0.0))
          .animate(CurvedAnimation(
    parent: _controllerAnimation,
    curve: Curves.ease,
  ));

  setLoading(status) {
    setState(() {
      isLoading = status;
    });
  }

  setVisibleButton() {
    visibleGetStartedButton =
        (currentIndex == widget.items.length - 1) ? true : false;
    visibleSkipOrNextButton =
        (currentIndex == widget.items.length - 1) ? false : true;
  }

  setCurrentIndex(index) {
    setLoading(true);
    _controllerAnimation.forward().then((val) {
      setState(() {
        currentIndex = index;
        setVisibleButton();
      });
    }).then((val) => {
          _controllerAnimation.reverse().then((val) {
            setLoading(false);
          })
        });
  }

  onNext() {
    if (currentIndex < widget.items.length - 1) {
      setCurrentIndex(currentIndex + 1);
    }
  }

  onSkip() {
    setCurrentIndex(widget.items.length - 1);
  }

  @override
  Widget build(BuildContext context) {
    HwIntroductionItem introductionItem = widget.items[currentIndex];

    Color nextButtonColorDefault = widget.nextButtonColor ?? HwIntroductionStyle.activeColor;
    Widget nextButtonDefault = widget.nextButton ??
        Text(
          'Next',
          textAlign: TextAlign.center,
          style: TextStyle(
            fontWeight: FontWeight.bold,
            color: Colors.white,
          ),
        );

    Color skipButtonColorDefault = widget.skipButtonColor ?? Colors.transparent;
    Widget skipButtonDefault = widget.skipButton ??
        Text(
          'Skip',
          textAlign: TextAlign.center,
          style: TextStyle(
            fontWeight: FontWeight.bold,
          ),
        );

    Color finishButtonColorDefault = widget.finishButtonColor ?? HwIntroductionStyle.activeColor;
    Widget finishButtonDefault = widget.finishButton ??
        Text(
          'Finish',
          textAlign: TextAlign.center,
          style: TextStyle(fontWeight: FontWeight.bold, color: Colors.white),
        );

    return Container(
      padding: HwIntroductionStyle.screenPadding,
      child: Column(mainAxisAlignment: MainAxisAlignment.end, children: [
        // Slider
        SlideTransition(
          position: _offsetAnimation,
          child: HwIntroductionItem(
            title: introductionItem.title,
            titleColor: introductionItem.titleColor,
            titleTextStyle: introductionItem.titleTextStyle,
            caption: introductionItem.caption,
            captionColor: introductionItem.captionColor,
            captionTextStyle: introductionItem.captionTextStyle,
            image: introductionItem.image,            
          ),
        ),

        SizedBox(
          height: HwIntroductionStyle().height(context) * .05,
        ),

        HwIntroductionIndicator(length: widget.items.length, currentIndex: currentIndex, activeColor: widget.indicatorActiveColor, unactiveColor: widget.indicatorUnactiveColor,),

        SizedBox(
          height: HwIntroductionStyle().height(context) * .05,
        ),

        // Skip or Next Button
        Visibility(
            visible: visibleSkipOrNextButton,
            child: Container(
              child: Row(
                children: [
                  Flexible(
                    child: HwIntroductionButton(
                      onTap: onSkip,
                      color: skipButtonColorDefault,
                      child: skipButtonDefault,
                    ),
                  ),
                  SizedBox(
                    height: HwIntroductionStyle().width(context) * .05,
                  ),
                  Flexible(
                    child: HwIntroductionButton(
                      onTap: onNext,
                      color: nextButtonColorDefault,
                      child: nextButtonDefault,
                    ),
                  )
                ],
              ),
            )),

        Visibility(
          visible: visibleGetStartedButton,
          child: Container(
            width: HwIntroductionStyle().width(context) * .8,
            child: HwIntroductionButton(
              onTap: () {
                if (widget.onFinish != null) {
                  widget.onFinish!();
                }
              },
              color: finishButtonColorDefault,
              child: finishButtonDefault,
            ),
          ),
        )
      ]),
    );
  }
}
